<?php
function pto_woocommerce_get_products($skip_filter) {
	$args = array(
		'post_type' => 'product',
		'posts_per_page' => -1,
		'post_status' => 'publish',
		'orderby' => 'title',
		'order' => 'ASC'
	);
	$products = get_posts($args);
	if(empty($skip_filter)) {
		$links = get_option('cqpim_wc_product_links');
		if(empty($links)) {
			$links = array();
		}
		foreach($products as $key => $product) {
			foreach($links as $link) {
				$product_id = isset($link['product']) ? $link['product'] : '';
				if($product_id == $product->ID) {
					unset($products[$key]);
				}
			}
		}
	}
	return $products;
}
function pto_woocommerce_get_templates() {
	$args = array(
		'post_type' => 'cqpim_templates',
		'posts_per_page' => -1,
		'post_status' => 'private',
		'orderby' => 'title',
		'order' => 'ASC'
	);
	$templates = get_posts($args);
	return $templates;
}
add_action( 'woocommerce_order_status_processing', 'pto_check_wc_order' );
function pto_check_wc_order($order_id){
    $order = wc_get_order($order_id);
	$order_products = array();
	foreach($order->get_items() as $item_id => $item_product){
		$order_products[] = $item_product->get_product_id();
		
	}
	$templates_to_add = array();
	$teams_to_add = array();
	$links = get_option('cqpim_wc_product_links');
	foreach($links as $link) {
		if(in_array($link['product'], $order_products)) {
			$templates_to_add[] = $link;
			$pto_order = true;
			update_post_meta($order_id, 'pto_order', $pto_order);
		}
		if(!empty($link['team_members'])) {
			$teams_to_add = $link['team_members'];
		}	
	}
	if(!empty($pto_order)) {
		$email = $order->get_billing_email();
		$data = array(
			'email' => $email,
			'first_name' => $order->get_billing_first_name(),
			'last_name' => $order->get_billing_last_name(),
			'company' => $order->get_billing_company(),
			'address1' => $order->get_billing_address_1(),
			'address2' => $order->get_billing_address_2(),
			'city' => $order->get_billing_city(),
			'state' => $order->get_billing_state(),
			'postcode' => $order->get_billing_postcode(),
			'phone' => $order->get_billing_phone(),
			'teams' => $teams_to_add,
		);
		$user = wp_get_current_user();
		if(empty($user->ID)) {
			$user = get_user_by('email', $email);
		}
		if(!empty($user->ID)) {
			if(in_array('cqpim_client', $user->roles)) {
				$client_id = pto_get_client_from_userid($user);
				$client_id = $client_id['assigned'];
				update_post_meta($order_id, '_customer_user', $user->ID);
				$project_pid = pto_woocommerce_create_project($client_id, $data, $order_id, $templates_to_add);
				update_post_meta($order_id, 'resulting_project', $project_pid);
				if(get_option('pto_wc_mark_complete')) {
					$order->update_status( 'completed' );
				}
			} else {
				$client_id = pto_woocommerce_convert_client($user->ID, $data);
				update_post_meta($order_id, '_customer_user', $user->ID);
				$project_pid = pto_woocommerce_create_project($client_id, $data, $order_id, $templates_to_add);
				update_post_meta($order_id, 'resulting_project', $project_pid);
				if(get_option('pto_wc_mark_complete')) {
					$order->update_status( 'completed' );
				}
			}
		} else {
			$client_array = pto_woocommerce_create_client($data);
			update_post_meta($order_id, '_customer_user', $client_array['uid']);
			$project_pid = pto_woocommerce_create_project($client_array['cid'], $data, $order_id, $templates_to_add);
			update_post_meta($order_id, 'resulting_project', $project_pid);
			if(get_option('pto_wc_mark_complete')) {
				$order->update_status( 'completed' );
			}		
		}
	} else {
	}
}
function pto_woocommerce_convert_client($user_id, $data) {
	$user = get_user_by('id', $user_id);
	$first_name = isset($data['first_name']) ? $data['first_name'] : '';
	$last_name = isset($data['last_name']) ? $data['last_name'] : '';
	$company = isset($data['company']) ? $data['company'] : '';
	$address1 = isset($data['address1']) ? $data['address1'] : '';
	$address2 = isset($data['address2']) ? $data['address2'] : '';
	$city = isset($data['city']) ? $data['city'] : '';
	$state = isset($data['state']) ? $data['state'] : '';
	$postcode = isset($data['postcode']) ? $data['postcode'] : '';
	$phone = isset($data['phone']) ? $data['phone'] : '';
	if(empty($company)) {
		$company = $first_name . ' ' . $last_name;
	}
	$name = $first_name . ' ' . $last_name;
	$address = $address1;
	if(!empty($address2)) {
		$address .= ', ' . $address2;
	}
	if(!empty($city)) {
		$address .= ', ' . $city;
	}
	if(!empty($state)) {
		$address .= ', ' . $state;
	}
	$new_client = array(
		'post_type' => 'cqpim_client',
		'post_status' => 'private',
		'post_content' => '',
		'post_title' => $company,
	);
	$client_pid = wp_insert_post( $new_client, true );	
	if( ! is_wp_error( $client_pid ) ){
		$client_updated = array(
			'ID' => $client_pid,
			'post_name' => $client_pid,
		);						
		wp_update_post( $client_updated );
		$client_details = array(
			'client_ref' => $client_pid,
			'client_company' => $company,
			'client_contact' => $user->display_name,
			'client_address' => $address,
			'client_postcode' => $postcode,
			'client_telephone' => $phone,
			'client_email' => $email,
		);
		$client_details['user_id'] = $user->ID;
		$client_ids = array();
		$client_ids[] = $user->ID;
		update_post_meta($client_pid, 'client_details', $client_details);
		update_post_meta($client_pid, 'client_ids', $client_ids);
		update_post_meta($client_pid, 'wc_client', true);
		$user_data = array(
			'ID' => $user->ID,
			'role' => 'cqpim_client'
		);
		wp_update_user($user_data);	
		return $client_pid;
	}
}
function pto_woocommerce_create_client($data) {
	$email = isset($data['email']) ? $data['email'] : '';
	$first_name = isset($data['first_name']) ? $data['first_name'] : '';
	$last_name = isset($data['last_name']) ? $data['last_name'] : '';
	$company = isset($data['company']) ? $data['company'] : '';
	$address1 = isset($data['address1']) ? $data['address1'] : '';
	$address2 = isset($data['address2']) ? $data['address2'] : '';
	$city = isset($data['city']) ? $data['city'] : '';
	$state = isset($data['state']) ? $data['state'] : '';
	$postcode = isset($data['postcode']) ? $data['postcode'] : '';
	$phone = isset($data['phone']) ? $data['phone'] : '';
	if(empty($company)) {
		$company = $first_name . ' ' . $last_name;
	}
	$name = $first_name . ' ' . $last_name;
	$address = $address1;
	if(!empty($address2)) {
		$address .= ', ' . $address2;
	}
	if(!empty($city)) {
		$address .= ', ' . $city;
	}
	if(!empty($state)) {
		$address .= ', ' . $state;
	}
	$new_client = array(
		'post_type' => 'cqpim_client',
		'post_status' => 'private',
		'post_content' => '',
		'post_title' => $company,
	);
	$client_pid = wp_insert_post( $new_client, true );
	if( ! is_wp_error( $client_pid ) ){
		$client_updated = array(
			'ID' => $client_pid,
			'post_name' => $client_pid,
		);						
		wp_update_post( $client_updated );
		$client_details = array(
			'client_ref' => $client_pid,
			'client_company' => $company,
			'client_contact' => $name,
			'client_address' => $address,
			'client_postcode' => $postcode,
			'client_telephone' => $phone,
			'client_email' => $email,
		);
		update_post_meta($client_pid, 'client_details', $client_details);
		$passw = pto_random_string(10);
		$login = $email;
		$user_id = wp_create_user( $login, $passw, $email );
		$user = new WP_User( $user_id );
		$user->set_role( 'cqpim_client' );
		$client_details = get_post_meta($client_pid, 'client_details', true);
		$client_details['user_id'] = $user_id;
		update_post_meta($client_pid, 'client_details', $client_details);
		$client_ids = array();
		$client_ids[] = $user_id;				
		update_post_meta($client_pid, 'client_ids', $client_ids);
		update_post_meta($client_pid, 'wc_client', true);
		$user_data = array(
			'ID' => $user_id,
			'display_name' => $name,
			'first_name' => $name,
		);
		wp_update_user($user_data);	
		$form_auto_welcome = get_option('form_reg_auto_welcome');
		if($form_auto_welcome == 1) {
			send_pto_welcome_email($client_pid, $passw);
		}	
		return array(
			'cid' => $client_pid,
			'uid' => $user_id,
		);
	} else {
		exit;	
	}			
}
function pto_woocommerce_create_project($client_id, $data, $order_id, $templates_to_add) {
	$email = isset($data['email']) ? $data['email'] : '';
	$first_name = isset($data['first_name']) ? $data['first_name'] : '';
	$last_name = isset($data['last_name']) ? $data['last_name'] : '';
	$company = isset($data['company']) ? $data['company'] : '';
	$address1 = isset($data['address1']) ? $data['address1'] : '';
	$address2 = isset($data['address2']) ? $data['address2'] : '';
	$city = isset($data['city']) ? $data['city'] : '';
	$state = isset($data['state']) ? $data['state'] : '';
	$postcode = isset($data['postcode']) ? $data['postcode'] : '';
	$phone = isset($data['phone']) ? $data['phone'] : '';
	$teams_to_add = isset($data['teams']) ? $data['teams'] : '';
	if(empty($company)) {
		$company = $first_name . ' ' . $last_name;
	}
	$name = $first_name . ' ' . $last_name;
	$address = $address1;
	if(!empty($address2)) {
		$address .= ', ' . $address2;
	}
	if(!empty($city)) {
		$address .= ', ' . $city;
	}
	if(!empty($state)) {
		$address .= ', ' . $state;
	}
	$client_details = get_post_meta($client_id, 'client_details', true);
	$client_contact = isset($client_details['user_id']) ? $client_details['user_id'] : '';
	$project_title = sprintf(__('%1$s - Project: Order %2$s', 'cqpim-woocommerce'), $company, $order_id);
	$new_project = array(
		'post_type' => 'cqpim_project',
		'post_status' => 'private',
		'post_content' => '',
		'post_title' => $project_title
	);
	$project_pid = wp_insert_post( $new_project, true );
	if( ! is_wp_error( $project_pid ) ){
		$project_updated = array(
			'ID' => $project_pid,
			'post_name' => $project_pid,
		);						
		wp_update_post( $project_updated );
		$project_details = array(
			'client_id' => $client_id,
			'quote_ref' => $order_id,
			'start_date' => current_time('timestamp'),
			'finish_date' => '',
			'pm_name' => '',
			'deposit_amount' => 0,
			'default_contract_text' => '',
			'quote_id' => '',
			'quote_type' => '',
			'client_contact' => $client_contact,
			'project_summary' => '',
		);
		update_post_meta($project_pid, 'project_details', $project_details);
		update_post_meta($project_pid, 'wc_project', true);
		update_post_meta($project_pid, 'contract_status', 2);
		if(!empty($teams_to_add)) {
			$team_members = array();
			foreach($teams_to_add as $member) {
				$team_members[] = array(
					'team_id' => $member,
					'pm' => 0,
				);
			}
			update_post_meta($project_pid, 'project_contributors', $team_members);
		}
		$project_progress = array();
		$project_progress[] = array(
			'update' => __('Project created', 'cqpim-woocommerce'),
			'date' => current_time('timestamp'),
			'by' => __('System', 'cqpim-woocommerce')
		);
		update_post_meta($project_pid, 'project_progress', $project_progress);
		if(pto_check_addon_status('bugs')) {
			$option = get_option('cqpim_bugs_auto');
			update_post_meta($project_pid, 'bugs_activated', $option);
		}
		$template_ids = array();
		foreach($templates_to_add as $template) {
			$template_ids[] = $template['template'];
		}
		$hid = 0;
		$hwe = 0;
		$elements = get_post_meta($project_pid, 'project_elements', true);
		$item_ref = isset($project_pid) ? $project_pid : '';
		foreach($template_ids as $id) {
			$template_contents = get_post_meta($id, 'project_template', true);
			if(!empty($template_contents)) {
				if(!empty($elements)) {
					$milestones = $elements;
				} else {
					$milestones = array();
				}
				if(!empty($hid)) {
					$i = $hid;
				} else {
					$i = 0;
				}
				foreach($template_contents['milestones'] as $milestone) {
					$mtitle = isset($milestone['title']) ? $milestone['title'] : '';
					$mdeadline = isset($milestone['deadline']) ? $milestone['deadline'] : '';
					$mstart = isset($milestone['start']) ? $milestone['start'] : '';
					$mcost = isset($milestone['cost']) ? $milestone['cost'] : '';
					if(!empty($hwe)) {
						$mweight = isset($milestone['weight']) ? $milestone['weight'] : 0;
						$mweight = $mweight + $hwe;
					} else {
						$mweight = isset($milestone['weight']) ? $milestone['weight'] : 0;
					}
					$milestones[$item_ref . '-' . $i] = array(
						'title' => $milestone['title'],
						'id' => $item_ref . '-' . $i,
						'deadline' => $milestone['deadline'],
						'start' => $milestone['start'],
						'cost' => $milestone['cost'],
						'weight' => $mweight,
					);
					if(!empty($milestone['tasks']['task_arrays'])) {
						foreach($milestone['tasks']['task_arrays'] as $task) {
							$title = isset($task['title']) ? $task['title'] : '';
							$description = isset($task['description']) ? $task['description'] : '';
							$start = isset($task['start']) ? $task['start'] : '';
							$deadline = isset($task['deadline']) ? $task['deadline'] : '';
							$weight = isset($task['weight']) ? $task['weight'] : '';
							$assignee = isset($task['assignee']) ? $task['assignee'] : '';
							$new_task = array(
								'post_type' => 'cqpim_tasks',
								'post_status' => 'publish',
								'post_content' => '',
								'post_title' => $title,
								'post_password' => pto_random_string(10),
							);
							$task_pid = wp_insert_post( $new_task, true );
							if( is_wp_error( $task_pid ) ){
								$return =  array( 
									'error' 	=> true,
									'errors' 	=> '<div class="cqpim-alert cqpim-alert-danger alert-display">' . __('There was a problem creating the tasks. Please try again later.', 'cqpim') . '</div>'
								);
								header('Content-type: application/json');
								echo json_encode($return);			
								exit();							
							} else {
								$task_updated = array(
									'ID' => $task_pid,
									'post_name' => $task_pid,
								);						
								wp_update_post( $task_updated );
								update_post_meta($task_pid, 'milestone_id', $item_ref . '-' . $i);	
								update_post_meta($task_pid, 'project_id', $item_ref);
								$task_details = array(
									'deadline' => $deadline,
									'status' => 'pending',
									'task_start' => $start,
									'task_description' => $description,
									'task_pc' => 0,
									'task_priority' => 'normal',
									'weight' => $weight,
								);
								update_post_meta($task_pid, 'task_details', $task_details);	
								if(!empty($assignee) && pto_is_team_on_project($item_ref, $assignee)) {
									update_post_meta($task_pid, 'owner', $assignee);
								}							
							}
							if(!empty($task['subtasks']['task_arrays'])) {
								foreach($task['subtasks']['task_arrays'] as $subtask) {
									$title = isset($subtask['title']) ? $subtask['title'] : '';
									$description = isset($subtask['description']) ? $subtask['description'] : '';
									$start = isset($subtask['start']) ? $subtask['start'] : '';
									$deadline = isset($subtask['deadline']) ? $subtask['deadline'] : '';
									$weight = isset($subtask['weight']) ? $subtask['weight'] : '';
									$assignee = isset($subtask['assignee']) ? $subtask['assignee'] : '';
									$new_subtask = array(
										'post_type' => 'cqpim_tasks',
										'post_status' => 'publish',
										'post_content' => '',
										'post_title' => $title,
										'post_parent' => $task_pid,
										'post_password' => pto_random_string(10),
									);
									$subtask_pid = wp_insert_post( $new_subtask, true );
									if( is_wp_error( $subtask_pid ) ){
										$return =  array( 
											'error' 	=> true,
											'errors' 	=> '<div class="cqpim-alert cqpim-alert-danger alert-display">' . __('There was a problem creating the tasks. Please try again later.', 'cqpim') . '</div>'
										);
										header('Content-type: application/json');
										echo json_encode($return);			
										exit();							
									} else {
										$task_updated = array(
											'ID' => $subtask_pid,
											'post_name' => $subtask_pid,
										);						
										wp_update_post( $task_updated );
										update_post_meta($subtask_pid, 'milestone_id', $item_ref . '-' . $i);	
										update_post_meta($subtask_pid, 'project_id', $item_ref);
										$task_details = array(
											'deadline' => $deadline,
											'status' => 'pending',
											'task_start' => $start,
											'task_description' => $description,
											'task_pc' => 0,
											'task_priority' => 'normal',
											'weight' => $weight,
										);
										update_post_meta($subtask_pid, 'task_details', $task_details);	
										if(!empty($assignee) && pto_is_team_on_project($item_ref, $assignee)) {
											update_post_meta($subtask_pid, 'owner', $assignee);
										}									
									}									
								}
							}
						}
					}
					$i++;
				}
				update_post_meta($project_pid, 'project_elements', $milestones);
			}			
		}
		pto_send_new_project_notification($project_pid);
		return $project_pid;
	}
}
function pto_send_new_project_notification($project_pid) {
	$subject = get_option('cqpim_wc_new_project_subject');
	$content = get_option('cqpim_wc_new_project_content');
}