<?php
// Project Bugs Metabox
function pto_project_bugs_metabox_callback( $post ) {
	// Add an nonce field so we can check for it later.
 	wp_nonce_field( 
 	 	'project_bugs_metabox', 
 	 	'project_bugs_metabox_nonce' ); 
		$active = get_post_meta($post->ID, 'bugs_activated', true); ?>
		<?php if(current_user_can('cqpim_activate_bugs')) { ?>
		<p><input type="checkbox" name="enable_bugs" value="1" <?php checked($active, 1); ?> /> <?php _e('Enable the Bug Tracker on this Project', 'cqpim-bugs'); ?></p>
		<br />
		<?php }
		if($active == 1) {
			$user = wp_get_current_user();
			if(current_user_can('cqpim_view_all_bugs')) {
				$args = array(
					'post_type' => 'cqpim_bug',
					'posts_per_page' => -1,
					'post_status' => 'private',
					'meta_key' => 'bug_project',
					'meta_value' => $post->ID,
				);
			} else {
				$args = array(
					'post_type' => 'cqpim_bug',
					'posts_per_page' => -1,
					'post_status' => 'private',
					'meta_query' => array(
						array(
							'key' => 'bug_project',
							'value' => $post->ID,
							'compare' => '=',
						),
						array(
							'key' => 'bug_assignee',
							'value' => pto_get_team_member_from_userid(),
							'compare' => '=',
						),
					),				
				);			
			}
			$bugs = get_posts($args);
			if(empty($bugs)) {
				echo '<p>' . __('There are no bugs raised for this project', 'cqpim-bugs') . '</p>';
			} else { ?>
				<table id="project_bugs_table" class="datatable_style">
					<thead>
						<tr>
							<th style="display:none"><?php _e('Updated Stamp', 'cqpim-bugs'); ?></th>
							<th><?php _e('Bug ID', 'cqpim-bugs'); ?></th>
							<th><?php _e('Bug Title', 'cqpim-bugs'); ?></th>
							<th><?php _e('Assignee', 'cqpim-bugs'); ?></th>
							<th><?php _e('Updated', 'cqpim-bugs'); ?></th>
							<th><?php _e('Status', 'cqpim-bugs'); ?></th>
						</tr>
					</thead>
					<tbody>
						<?php foreach($bugs as $bug) { 
							$assignee = get_post_meta($bug->ID, 'bug_assignee', true);
							$updated = get_post_meta($bug->ID, 'bug_updated', true);
							$team_details = get_post_meta($assignee, 'team_details', true);
							$team_name = isset($team_details['team_name']) ? $team_details['team_name'] : '';
							$status = get_post_meta($bug->ID, 'bug_status', true);
							$statuses = pto_get_bug_status();
							?>
							<tr>
								<td style="display:none"><?php echo $updated; ?></td>
								<td><a href="<?php echo get_edit_post_link($bug->ID); ?>"><?php echo $bug->ID; ?></a></td>
								<td><a href="<?php echo get_edit_post_link($bug->ID); ?>"><?php echo $bug->post_title; ?></a></td>
								<td><?php echo $team_name; ?></td>
								<td><?php echo date(get_option('cqpim_date_format') . ' H:i:s', $updated); ?></td>
								<td><span class="<?php echo $statuses[$status]['class']; ?>"><?php echo $statuses[$status]['name']; ?></span></td>
							</tr>
						<?php } ?>
					</tbody>
				</table>
			<?php }
		} else {
			echo '<p>' . __('The Bug Tracker is disabled on this project', 'cqpim-bugs') . '</p>';
		}
		if(current_user_can('cqpim_create_new_bug') && !empty($active)) { ?>
			<button id="create_bug_trigger" class="cqpim_button cqpim_small_button right mt-20 font-blue border-blue right"><?php _e('Raise New Bug', 'cqpim-bugs'); ?></button>
			<div class="clear"></div>
			<div id="password_reset_container" style="display:none">
				<div id="password_reset">
					<div style="padding:12px">
						<h3><?php _e('Raise New Bug', 'cqpim'); ?></h3>
						<input type="text" id="bug_title" placeholder="<?php _e('Bug Title', 'cqpim-bugs'); ?>" />
						<br /><br />
						<textarea style="height:100px" id="bug_desc" placeholder="<?php _e('Bug Description', 'cqpim-bugs'); ?>"></textarea>
						<br /><br />
						<select style="margin-right:20px" id="bug_assignee" name="bug_assignee">
							<option value="0"><?php _e('Choose an Assignee', 'cqpim-bugs'); ?></option>
							<?php 
							$members = pto_bugs_get_bug_assignees($post->ID); 
							if(!empty($members)) {
								foreach($members as $member) {
									$member_details = get_post_meta($member['team_id'], 'team_details', true);
									if(!empty($member_details['user_id']) && !empty($member_details['team_name'])) {
										echo '<option value="' . $member['team_id'] . '">' . $member_details['team_name'] . '</option>';
									}
								}
							}
							?>
						</select>
						<select id="bug_status" name="bug_status">
							<option value=""><?php _e('Choose a Status', 'cqpim-bugs'); ?></option>
							<?php
							$statuses = pto_get_bug_status();
							if(!empty($statuses)) {
								foreach($statuses as $key => $status) {
									echo '<option class="' . $status['class'] . '" value="' . $key . '">' . $status['name'] . '</option>';
								}
							}
							?>
						</select>
						<div class="clear"></div>
						<br /><br />
						<input type="file" class="cqpim-file-upload" name="async-upload" id="attachments" multiple />
						<div id="upload_attachments"></div>
						<div class="clear"></div>
						<input type="hidden" name="image_id" id="upload_attachment_ids">
						<br /><br />
						<div id="password_messages"></div>
						<button class="cancel-colorbox cqpim_button font-red border-red op"><?php _e('Cancel', 'cqpim'); ?></button>
						<button id="raise_bug_ajax" class="cqpim_button font-green border-green right op" value="<?php echo $client_user; ?>"><?php _e('Raise New Bug', 'cqpim'); ?></button>
						<div class="clear"></div>
					</div>
				</div>
			</div>
		<?php }
}
// Save Notes Details
add_action( 'save_post', 'save_pto_project_bugs_metabox_data' );
function save_pto_project_bugs_metabox_data( $post_id ){
	//  verify this came from the our screen and with proper authorization
	// Check if our nonce is set.
	if ( ! isset( $_POST['project_bugs_metabox_nonce'] ) )
	    return $post_id;
	// Verify that the nonce is valid.
	$nonce = $_POST['project_bugs_metabox_nonce'];
	if ( ! wp_verify_nonce( $nonce, 'project_bugs_metabox' ) )
	    return $post_id;
	// If this is an autosave, our form has not been submitted, so we don't want to do anything.
	if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) 
	    return $post_id;
	// Check the user's permissions.
	if ( 'page' == $_POST['post_type'] ) {
	    if ( ! current_user_can( 'edit_page', $post_id ) )
	        return $post_id;
	  	} else {
	    if ( ! current_user_can( 'edit_post', $post_id ) )
	        return $post_id;
	}
	if(current_user_can('cqpim_activate_bugs')) {
		$bugs_active = isset($_POST['enable_bugs']) ? $_POST['enable_bugs'] : 0;
		update_post_meta($post_id, 'bugs_activated', $bugs_active);
	}
}	