<?php
function pto_bugs_get_bug_assignees($post_id) {
	$team_members = array();
	$project_contributors = get_post_meta($post_id, 'project_contributors', true);
	if(!empty($project_contributors)) {
		foreach($project_contributors as $contributor) {
			$team_details = get_post_meta($contributor['team_id'], 'team_details', true);
			if(!empty($team_details['user_id'])) {
				$user = get_user_by('id', $team_details['user_id']);
				if(user_can($user, 'cqpim_view_bugs')) {
					$team_members[] = $contributor;
				}
			}
		}
	}
	return $team_members;
}
add_action( "wp_ajax_nopriv_pto_update_bug", "pto_update_bug");
add_action( "wp_ajax_pto_update_bug", "pto_update_bug");	
function pto_update_bug($bug_pid, $title = NULL, $project = NULL, $desc = NULL, $assignee = NULL, $status = NULL, $update = NULL, $files = array(), $user = NULL) {
	if(!empty($_POST['bug_id']) || !empty($_POST['new'])) {
		$ajax = true;
		$project = isset($_POST['project']) ? $_POST['project'] : '';
		$bug_pid = isset($_POST['bug_id']) ? $_POST['bug_id'] : '';
		$title = isset($_POST['title']) ? $_POST['title'] : '';
		$desc = isset($_POST['desc']) ? $_POST['desc'] : '';
		$assignee = isset($_POST['assignee']) ? $_POST['assignee'] : '';
		$status = isset($_POST['status']) ? $_POST['status'] : '';
		$update = isset($_POST['update']) ? $_POST['update'] : '';
		$new = isset($_POST['new']) ? $_POST['new'] : 0;
		$files = isset($_POST['files']) ? $_POST['files'] : array();
	}
	if(empty($user)) {
		$user = wp_get_current_user();
	}
	if(!empty($ajax) && !empty($new) && empty($project) || !empty($ajax) && empty($title) || !empty($ajax) && empty($new) && empty($desc) || !empty($ajax) && empty($assignee) || !empty($ajax) && empty($status)) {
		$return =  array( 
			'error' 	=> true,
			'message' 	=> '<div class="cqpim-alert cqpim-alert-danger alert-display">' . __('Please enter a Title, Description, Assignee and Status.', 'cqpim-bugs') . '</div>',
		);
		header('Content-type: application/json');
		echo json_encode($return);
		exit();
	} else {
		if(!empty($new)) {
			// Create the bug
			$new_bug = array(
				'post_type' => 'cqpim_bug',
				'post_status' => 'private',
				'post_content' => '',
				'post_title' =>  $title,
			);
			$bug_pid = wp_insert_post( $new_bug, true );
			if( is_wp_error( $bug_pid ) ){
				$return =  array( 
					'error' 	=> true,
					'message' 	=> '<div class="cqpim-alert cqpim-alert-danger alert-display">' . __('The bug could not be created, please try again.', 'cqpim-bugs') . '</div>',
				);
				header('Content-type: application/json');
				echo json_encode($return);
				exit();					
			}
			// Add an Update
			$bug_updates = array();
			$bug_updates[] = array(			
				'update' => '',
				'stamp' => current_time('timestamp'),
				'user' => $user->ID,
				'changes' => array(),
				'status' => $status,
				'files' => explode(',', $files),
			);	
			update_post_meta($bug_pid, 'bug_project', $project);
			pto_add_team_notification($assignee, $user->ID, $bug_pid, 'bug_assigned');
			$team_members = pto_bugs_get_bug_assignees($project);
			if(!empty($team_members)) {
				foreach($team_members as $member) {
					pto_add_team_notification($member['team_id'], $user->ID, $bug_pid, 'bug_new');
				}
			}
		} else {
			// Log changes to the bug
			$changes = array();
			$oassignee = get_post_meta($bug_pid, 'bug_assignee', true);
			$ostatus = get_post_meta($bug_pid, 'bug_status', true);
			if($assignee != $oassignee) {
				$changes[] = array(
					'type' => 'a',
					'from' => $oassignee,
					'to' => $assignee,
				);
				pto_add_team_notification($assignee, $user->ID, $bug_pid, 'bug_assigned');
			}
			$team_details = get_post_meta($assignee, 'team_details', true);
			$user_id = isset($team_details['user_id']) ? $team_details['user_id'] : '';
			if($user_id != $user->ID) {
				pto_add_team_notification($assignee, $user->ID, $bug_pid, 'bug_updated');
			}
			if($status != $ostatus) {
				$changes[] = array(
					'type' => 's',
					'from' => $ostatus,
					'to' => $status,
				);
			}
			$bug_updates = get_post_meta($bug_pid, 'bug_updates', true);
			$bug_updates[] = array(			
				'update' => $update,
				'stamp' => current_time('timestamp'),
				'user' => $user->ID,
				'changes' => $changes,
				'status' => $status,
				'files' => explode(',', $files),
			);
		}
		// Update Post Meta
		update_post_meta($bug_pid, 'bug_updates', $bug_updates);
		update_post_meta($bug_pid, 'bug_assignee', $assignee);
		update_post_meta($bug_pid, 'bug_status', $status);
		update_post_meta($bug_pid, 'bug_desc', $desc);
		update_post_meta($bug_pid, 'bug_updated', current_time('timestamp'));
		// Own Files
		$attachment_links = array();
		if(!empty($files)) {
			$file_arr = explode(',', $files);
			foreach($file_arr as $attachment) {
				$attachment_updated = array(
					'ID' => $attachment,
					'post_parent' => $bug_pid,
				);
				wp_update_post($attachment_updated);
				update_post_meta($attachment, 'cqpim', true);		
				$attachment_links[] = get_attached_file( $attachment );					
			}
		}
		// Send Emails (PM, Assignee, Client)
		$emails_to_send = array();
		// Should we email the client?
		$project_details = get_post_meta($project, 'project_details', true);
		$client_id = isset($project_details['client_id']) ? $project_details['client_id'] : '';
		$client_contact = isset($project_details['client_contact']) ? $project_details['client_contact'] : '';
		$client_contacts = get_post_meta($client_id, 'client_contacts', true);
		$client_details = get_post_meta($client_id, 'client_details', true);
		if(!empty($client_contact)) {
			if(!empty($client_details['user_id']) && $client_details['user_id'] == $client_contact) {
				$client_user = isset($client_details['user_id']) ? $client_details['user_id'] : '';
			} else {
				$client_user = isset($client_details[$client_contact]['user_id']) ? $client_details[$client_contact]['user_id'] : '';		
			}
		} else {
			$client_user = isset($client_details['user_id']) ? $client_details['user_id'] : '';	
		}			
		$client_usr_obj = get_user_by('id', $client_user);
		if($client_usr_obj->ID != $user->ID) {
			update_post_meta($bug_pid, 'updated', true);
			pto_add_team_notification($client_id, $user->ID, $bug_pid, 'bug_updated');
		}
		if(!empty($client_usr_obj)) {
			// NEED TO GET PREFERENCES AND DECIDE WHETHER TO INCLUDE
			$args = array(
				'post_type' => 'cqpim_client',
				'posts_per_page' => -1,
				'post_status' => 'private'
			);
			$members = get_posts($args);
			foreach($members as $member) {
				$team_details = get_post_meta($member->ID, 'client_details', true);
				if($team_details['user_id'] == $client_usr_obj->ID) {
					$notifications = get_post_meta($member->ID, 'client_notifications', true);
				}
			}
			if(empty($assigned)) {
				foreach($members as $member) {
					$client_contacts = get_post_meta($member->ID, 'client_contacts', true);
					if(empty($client_contacts)) {
						$client_contacts = array();
					}
					foreach($client_contacts as $contact) {
						if($contact['user_id'] == $client_usr_obj->ID) {
							$notifications = isset($contact['notifications']) ? $contact['notifications'] : array();
						}
					}
				} 			
			}
			$no_bugs = isset($notifications['no_bugs']) ? $notifications['no_bugs']: 0;
			$no_bugs_comment = isset($notifications['no_bugs_comment']) ? $notifications['no_bugs_comment']: 0;			
			if(!empty($no_bugs) || !empty($no_bugs_comment) && empty($update)) {
				// Do nothing
			} else {
				$emails_to_send[] = $client_usr_obj->ID;
			}
		}
		// Get Project Managers
		$project_contributors = get_post_meta($project, 'project_contributors', true);
		if(!empty($project_contributors)) {
			foreach($project_contributors as $contributor) {
				if(!empty($contributor['pm'])) {
					$team_details = get_post_meta($contributor['team_id'], 'team_details', true);
					$user_id = isset($team_details['user_id']) ? $team_details['user_id'] : '';	
					$emails_to_send[] = $user_id;
					if($user_id != $user->ID) {
						pto_add_team_notification($contributor['team_id'], $user->ID, $bug_pid, 'bug_updated');
					}
				}
			}
		}
		// Get Assignee
		$team_details = get_post_meta($assignee, 'team_details', true);
		$user_id = isset($team_details['user_id']) ? $team_details['user_id'] : '';
		$emails_to_send[] = $user_id;
		$emails_to_send = array_unique($emails_to_send);
		foreach($emails_to_send as $email) {
			if($new == 1) {
				$subject = get_option('cqpim_new_bug_subject');
				$content = get_option('cqpim_new_bug_content');
			} else {
				$subject = get_option('cqpim_update_bug_subject');
				$content = get_option('cqpim_update_bug_content');			
			}
			// Get Data
			$to_user = get_user_by('id', $email);
			$to_name = $to_user->display_name;
			$to = $to_user->user_email;
			$project_obj = get_post($project);
			$statuses = pto_get_bug_status();
			$as_details = get_post_meta($assignee, 'team_details', true);
			$as_user = get_user_by('id', $as_details['user_id']);
			$subject = str_replace('%%PIPING_ID%%', '[' . get_option('cqpim_string_prefix') . ':' . $bug_pid . ']', $subject);
			$subject = str_replace('%%CURRENT_USER%%', $user->display_name, $subject);
			$subject = str_replace('%%BUG_TITLE%%', $title, $subject);
			$content = str_replace('%%CURRENT_USER%%', $user->display_name, $content);
			$content = str_replace('%%NAME%%', $to_name, $content);
			$content = str_replace('%%NAME%%', $to_name, $content);				
			$content = str_replace('%%BUG_TITLE%%', $title, $content);
			$content = str_replace('%%PROJECT_TITLE%%', $project_obj->post_title, $content);
			$content = str_replace('%%BUG_STATUS%%', $statuses[$status]['name'], $content);
			$content = str_replace('%%BUG_ASSIGNEE%%', $as_user->display_name, $content);
			$content = str_replace('%%BUG_DESCRIPTION%%', $desc, $content);
			$content = str_replace('%%BUG_UPDATE%%', $update, $content);
			$subject = pto_replacement_patterns($subject, $bug_pid, 'company');
			$content = pto_replacement_patterns($content, $bug_pid, 'company');
			if($email != $user->ID) {
				pto_send_emails( $to, $subject, $content, '', $attachment_links, 'support' );
			}
		}
		if(!empty($ajax)) {
			$current_user = $user->display_name;
			if($new == 1) {
				$project_progress = get_post_meta($project_obj->ID, 'project_progress', true);
				$project_progress[] = array(
					'update' => __('New Bug Created', 'cqpim-bugs') . ': ' . $title,
					'date' => current_time('timestamp'),
					'by' => $current_user
				);
				update_post_meta($project_obj->ID, 'project_progress', $project_progress );
				$return =  array( 
					'error' 	=> false,
					'message' 	=> '<div class="cqpim-alert cqpim-alert-success alert-display">' . __('Bug added successfully.', 'cqpim-bugs') . '</div>',
					'redirect' => get_the_permalink($bug_pid),
				);
				header('Content-type: application/json');
				echo json_encode($return);
				exit();	
			} else {
				$project_progress = get_post_meta($project_obj->ID, 'project_progress', true);
				$project_progress[] = array(
					'update' => __('Bug Updated', 'cqpim-bugs') . ': ' . $title,
					'date' => current_time('timestamp'),
					'by' => $current_user
				);
				update_post_meta($project_obj->ID, 'project_progress', $project_progress );
				$return =  array( 
					'error' 	=> false,
					'message' 	=> '<div class="cqpim-alert cqpim-alert-success alert-display">' . __('Bug updated successfully.', 'cqpim-bugs') . '</div>',
				);
				header('Content-type: application/json');
				echo json_encode($return);
				exit();						
			}
		} else {
			if($new == 1) {
				$current_user = $user->display_name;
				$project_progress = get_post_meta($project_obj->ID, 'project_progress', true);
				$project_progress[] = array(
					'update' => __('New Bug Created', 'cqpim-bugs') . ': ' . $title,
					'date' => current_time('timestamp'),
					'by' => $current_user
				);
				update_post_meta($project_obj->ID, 'project_progress', $project_progress );
			} else {
				$current_user = wp_get_current_user();
				$current_user = $current_user->display_name;
				$project_progress = get_post_meta($project_obj->ID, 'project_progress', true);
				$project_progress[] = array(
					'update' => __('Bug Updated', 'cqpim-bugs') . ': ' . $title,
					'date' => current_time('timestamp'),
					'by' => $current_user
				);
				update_post_meta($project_obj->ID, 'project_progress', $project_progress );						
			}				
		}
	}
}
function pto_get_bug_status() {
	$bug_status = array();
	$bug_status['new'] = array(
			'name' => __('New', 'cqpim-bugs'),
			'class' => 'cqpim_button cqpim_small_button font-amber border-amber op nolink',
			'sclass' => 'font-white bg-amber op rounded_4'
	);
	$bug_status['progress'] = array(
			'name' => __('In Progress', 'cqpim-bugs'),
			'class' => 'cqpim_button cqpim_small_button font-blue border-blue op nolink',
			'sclass' => 'font-white bg-blue op rounded_4'
	);
	$bug_status['resolved'] = array(
			'name' => __('Resolved', 'cqpim-bugs'),
			'class' => 'cqpim_button cqpim_small_button font-green-sharp border-green-sharp op nolink',
			'sclass' => 'font-white bg-green-sharp op rounded_4'
	);
	$bug_status['rejected'] = array(
			'name' => __('Rejected', 'cqpim-bugs'),
			'class' => 'cqpim_button cqpim_small_button font-purple border-purple op nolink',
			'sclass' => 'font-white bg-purple op rounded_4'
	);
	$bug_status['closed'] = array(
			'name' => __('Closed', 'cqpim-bugs'),
			'class' => 'cqpim_button cqpim_small_button font-grey-cascade border-grey-cascade op nolink',
			'sclass' => 'font-white bg-grey-cascade op rounded_4'
	);
	return $bug_status;
}
add_action( "wp_ajax_nopriv_pto_delete_bug_page", 
		"pto_delete_bug_page");
add_action( "wp_ajax_pto_delete_bug_page", 
		"pto_delete_bug_page");
function pto_delete_bug_page() {
	$task_id = isset($_POST['task_id']) ? $_POST['task_id'] : '';
	$project_id = get_post_meta($task_id, 'bug_project', true);
	wp_delete_post($task_id, true);
	$return =  array( 
		'error' 	=> false,
		'redirect' 	=> admin_url() . 'post.php?post=' . $project_id . '&action=edit'
	);
	header('Content-type: application/json');
	echo json_encode($return);	
	exit();
}
function pto_populate_dashboard_bugs() { ?>
	<div class="cqpim_block">
		<div class="cqpim_block_title">
			<div class="caption">
				<i class="fa fa-bug font-green-sharp" aria-hidden="true"></i>
				<span class="caption-subject font-green-sharp sbold"><?php _e('Open Bugs', 'cqpim-bugs'); ?> </span>
			</div>
		</div>
		<?php $user = wp_get_current_user();
		if(current_user_can('cqpim_view_all_bugs')) {
			$args = array(
				'post_type' => 'cqpim_bug',
				'posts_per_page' => -1,
				'post_status' => 'private',
			);
		} else {
			$args = array(
				'post_type' => 'cqpim_bug',
				'posts_per_page' => -1,
				'post_status' => 'private',
				'meta_query' => array(
					array(
						'key' => 'bug_assignee',
						'value' => pto_get_team_from_userid(),
						'compare' => '=',
					),
				),				
			);			
		}
		$bugs = get_posts($args);
		if(empty($bugs)) {
			echo '<p>' . __('You do not have any open bugs.', 'cqpim-bugs') . '</p>';
		} else { ?>
			<table class="datatable_style dataTable" data-ordering="false" data-rows="5">
				<thead>
					<tr>
						<th style="display:none"><?php _e('Updated Stamp', 'cqpim-bugs'); ?></th>
						<th><?php _e('Bug Title', 'cqpim-bugs'); ?></th>
						<th><?php _e('Project', 'cqpim-bugs'); ?></th>
						<th><?php _e('Updated', 'cqpim-bugs'); ?></th>
						<th><?php _e('Status', 'cqpim-bugs'); ?></th>
					</tr>
				</thead>
				<tbody>
					<?php foreach($bugs as $bug) { 
						$assignee = get_post_meta($bug->ID, 'bug_assignee', true);
						$updated = get_post_meta($bug->ID, 'bug_updated', true);
						$team_details = get_post_meta($assignee, 'team_details', true);
						$team_name = isset($team_details['team_name']) ? $team_details['team_name'] : '';
						$status = get_post_meta($bug->ID, 'bug_status', true);
						$statuses = pto_get_bug_status();
						$project = get_post_meta($bug->ID, 'bug_project', true);
						$project_obj = get_post($project);
						if($status != 'closed') {
						?>
						<tr>
							<td style="display:none"><?php echo $updated; ?></td>
							<td><a href="<?php echo get_edit_post_link($bug->ID); ?>"><?php echo $bug->post_title; ?></a></td>
							<td><a href="<?php echo get_edit_post_link($project); ?>"><?php echo $project_obj->post_title; ?></a></td>
							<td data-sort="<?php echo $updated; ?>"><?php echo date(get_option('cqpim_date_format') . ' H:i:s', $updated); ?></td>
							<td><span class="<?php echo $statuses[$status]['class']; ?>"><?php echo $statuses[$status]['name']; ?></span></td>
						</tr>
						<?php } 
					} ?>
				</tbody>
			</table>
		<?php } ?>
	</div>
	<?php
}